<?php

namespace CodeClouds\Unify\Models;

use CodeClouds\Unify\Service\Request;
use \CodeClouds\Unify\Data_Sources\Connection_Handler;
use \CodeClouds\Unify\Model\Connection;
use CodeClouds\Unify\Service\Mapping\Fields;

/**
 * Payment model.
 * @package CodeClouds\Unify
 */
class Unify_Payment extends \WC_Payment_Gateway
{
    /**
     * @var String 
     */
    public $domain;

    /**
     * @var array 
     */
    private $payload = [];

    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        $this->domain = 'CodeClouds_Unify';

        $this->id                 = 'codeclouds_unify';
        $this->icon               = apply_filters('woocommerce_unify_gateway_icon', '');
        $this->has_fields         = false;
		$this->supports           = ['subscriptions', 'products'];
        $this->method_title       = __('Unify', $this->domain);
        $this->method_description = __('Accepts payments via LimeLight/Konnektive CRM and many more.', $this->domain);

        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        // Define user set variables
        $this->title       = $this->get_option('title');
        $this->description = $this->get_option('description');

        // Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_thankyou_unify', array($this, 'thankyou_page'));

        // Customer Emails
        add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
    }

    /**
     * Initialize Gateway Settings Form Fields.
     */
    public function init_form_fields()
    {
		if (!empty($_GET['page']) && $_GET['page'] == 'wc-settings' && !empty($_GET['section']) && $_GET['section'] == 'codeclouds_unify')
		{
			wp_redirect(admin_url('admin.php?page=unify-settings'), 302, 'Unify');
			die();
		}

		$this->form_fields = [
            'enabled'     => [
                'title'   => __('Enable', $this->domain),
                'type'    => 'checkbox',
                'label'   => __('Enable Unify', $this->domain),
                'default' => 'yes',
            ],
            'title'       => [
                'title'       => __('Title', $this->domain),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', $this->domain),
                'default'     => __('Unify Payment', $this->domain),
                'desc_tip'    => true,
            ],
            'description' => [
                'title'       => __('Description', $this->domain),
                'type'        => 'text',
                'description' => __('This controls the description which the user sees during checkout.', $this->domain),
                'default'     => __('Pay with your credit card.', $this->domain),
                'desc_tip'    => true,
            ],
            'connection'  => [
                'title'       => __('Connections', $this->domain),
                'type'        => 'select',
                'class'       => 'wc-enhanced-select',
                'description' => __('Choose a connection for payment process.', $this->domain),
                'default'     => '',
                'desc_tip'    => true,
                'options'     => \CodeClouds\Unify\Model\Connection::getArray(),
            ],
			'shipment_price_settings' => [
				'title' => __('Shipment Price Settings', $this->domain),
				'type'        => 'select',
                'class'       => 'wc-enhanced-select',
                'description' => __('If you have various shipping method for a product then you can turn on this feature. In Single Order option, all the product will be clubbed under one order and the shipping will be charged based on WooCommerce Shipping calculation and will reflect in one product price and with free shipping ID. In Multi-Order all the product will be clubbed under unique Shipping ID(s) as a result more than 1 order will be in generated in Limelight.', $this->domain),
                'default'     => '',
                'desc_tip'    => true,
                'options'     => ['' => 'Please chosse an option', 1 => 'Single order with custom product', 2 => 'Multiple orders'],
			],
			'shipping_product_id' => [
				'title' => __('Shipping Product ID', $this->domain),
				'type' => 'text',
				'description' => __('Limelight Product ID for Shipping Charge.', $this->domain),
				'default' => '',
				'desc_tip' => true,
			],
			'testmode'    => [
                'title'       => __('Test Mode', $this->domain),
                'type'        => 'checkbox',
                'description' => __('It will disable card number\'s validation.', $this->domain),
                'label'       => __('Enable Test Mode', $this->domain),
                'default'     => 'no',
            ],
        ];
    }

    /**
     * Output for the order received page.
     */
    public function thankyou_page()
    {
        if ($this->instructions)
        {
            echo wpautop(wptexturize($this->instructions));
        }
    }

    /**
     * Add content to the WC email.
     * @access public
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     */
    public function email_instructions($order, $sent_to_admin, $plain_text = false)
    {
        if ($this->instructions && !$sent_to_admin && 'CodeClouds_Unify' === $order->payment_method && $order->has_status('on-hold'))
        {
            echo wpautop(wptexturize($this->instructions)) . PHP_EOL;
        }
    }

    public function payment_fields()
    {
        if (!empty($this->settings['connection']))
        {
            if ($this->get_description())
            {
                echo wpautop(wptexturize($this->get_description()));
            }
            include_once __DIR__ . '/../Templates/cc_payment.php';
        }
        else
        {
            echo 'Plugin is not configured yet.';
        }
    }

    /**
     * Process the payment and return the result.
     * @param int $order_id
     * @return array
     */
    public function process_payment($order_id)
    {
        $order = \wc_get_order($order_id);

        $this->prepare_payload($order);
        
        /**
         * Payment by Connection.
         */
        $response = Connection_Handler::call($this->payload)->order();
		
        if(!isset($response['order_id']) || empty($response['order_id']))
        {
            throw new \Exception('Payment Failed! Please make sure you have entered the correct information.');
        }

        /**
         * Store payment information into order.
         */
        $order->update_meta_data('_codeclouds_unify_order_id', $response['order_id']);
        $order->update_meta_data('_codeclouds_unify_transaction_id', $response['transaction_id']);
        $order->update_meta_data('_codeclouds_unify_connection', $this->payload['config']['connection']);
        $order->update_meta_data('_codeclouds_unify_connection_id', $this->settings['connection']);
        
        $order->payment_complete($response['order_id']);
        $order->add_order_note(sprintf(__('%s payment approved! Transaction ID: %s', 'woocommerce'), $this->title, $response['order_id']));
		
		if (!empty($response['custom_note']) && is_array($response['custom_note']))
		{
			foreach ($response['custom_note'] as $val)
			{
				$order->add_order_note(sprintf(__('Transaction ID: %s has been declined.[%s]', 'woocommerce'), $val['order_id'], $val['message']));
			}
		}

		// Set order status
        // $order->update_status('wc-processing', __('Checkout with Unify Payment. ', $this->domain));

        // Reduce stock levels
        // $order->reduce_order_stock();

        // Remove cart
        WC()->cart->empty_cart();

        // Return thankyou redirect
        return [
            'result'   => 'success',
            'redirect' => $this->get_return_url($order),
        ];
    }

    /**
     * @param Object $order
     */
    private function prepare_payload($order)
    {
        $connection    = Connection::get_post_meta($this->settings['connection']);
        $expiry        = explode('/', Request::getPost('cc_expiry'));
        $this->payload = [
            'config'      => [
                'connection'   => $connection['unify_connection_crm'][0],
                'api_username' => $connection['unify_connection_api_username'][0],
                'api_password' => \Codeclouds\Unify\Model\Protection\Decryption::make($connection['unify_connection_api_password'][0], $connection['unify_connection_salt'][0]),
                'campaign_id'  => $connection['unify_connection_campaign_id'][0],
                'shipping_id'  => $connection['unify_connection_shipping_id'][0],
                'endpoint'     => $connection['unify_connection_endpoint'][0],
				'offer_model'  => !empty($connection['unify_connection_offer_model'][0]) ? $connection['unify_connection_offer_model'][0] : 0
            ],
            'card'        => [
                'type'      => Request::getPost('cc_type'),
                'number'    => Request::getPost('cc_number'),
                'exp_month' => $expiry[0],
                'exp_year'  => $expiry[1],
                'cvc'       => Request::getPost('cc_cvc')
            ],
            'billing'     => $order->get_address(),
            'shipping'    => $order->get_address('shipping'),
            'ip_address'  => $order->get_customer_ip_address(),
            'description' => sprintf('Payment from: %s. Order ID: #%s, %s', get_site_url(), $order->get_id(), $order->get_billing_email())
        ];
        
        if(!empty($order->get_customer_note()))
        {
            $this->payload['description'] = $this->payload['description'] . '. Note: ' . $order->get_customer_note();
        }

        $this->get_cart_items($order);
        $this->prepare_billing_shipping($order);
		$this->set_konnektive_shipping($connection, $order);
    }

	private function get_cart_items($order)
	{
		$cart_items = $order->get_items();
		$counter = 0;

		foreach ($cart_items as $cart_item)
		{
			//$connection = \CodeClouds\Unify\Actions\Product::get_connetion($cart_item['product_id']);
			$this->payload['cart_items'][$counter] = [
				'product_id' => $cart_item['product_id'],
				'price' => $order->get_item_total($cart_item, false, false),
				'qty' => $cart_item['qty']
			];

			foreach (Fields::get() as $field)
			{
				$this->payload['cart_items'][$counter][$field['payload']] = get_post_meta($cart_item['product_id'], $field['id'], true);
			}

			foreach ($cart_item->get_meta_data() as $variant)
			{

				$this->payload['cart_items'][$counter]['variants'][] = [
					'id' => $variant->get_data()['id'],
					'key' => str_replace('pa_', '', $variant->get_data()['key']),
					'value' => $variant->get_data()['value'],
				];
			}

			$counter ++;
		}
	}

	/**
     * Check shipping address. 
     * If shipping empty, assign shipping as billing.
     */
    private function prepare_billing_shipping($order)
    {
        if(!$order->has_shipping_address())
        {
            $this->payload['shipping'] = $this->payload['billing'];
        }
    }
	
	/**
	 * Setting Order Shipping Price in Konnektive cart item payload
	 * 
	 * @param array $connection
	 * @param Object $order WooCommerce Order Object
	 */
    private function set_konnektive_shipping($connection, $order)
    {
		//If CRM is Konnektive and the shipProfileId is empty then check order shipping price 
        if(!empty($connection['unify_connection_crm'][0]) && $connection['unify_connection_crm'][0] == 'konnektive' && empty($connection['unify_connection_shipping_id'][0]))
        {
			foreach ($this->payload['cart_items'] as $key => $v)
			{
				$this->payload['cart_items'][$key]['shipPrice'] = $key == 0 ? wc_format_decimal($order->get_total_shipping(), 2) : 0;
			}
        }
    }
}
